/*
 * This class is responsible for keeping the client up to date about changes to the user's account (new tunnels, newly logged on 
 * confidants, etc.).  It is not, however, responsible for keeping the client up to date about changes to the tunnel she is currently
 * logged into (such as new messages, etc.).
 */

function _AccountUpdateRetriever () {
	this.lastUpdateIdReceived=0; // an int
	this._running=false; // true if start() has been called, false if stop() has been called
	this.updateIntervalMillis=10000; // we check for new updates every 10 secs (much less often than for "regular" tunnel updates)
	this.requestUpdateTimeout=null;
	this.lastUpdateDataReceived={};
	this.lastCallWasSingleCall=false;
}

var p=_AccountUpdateRetriever.prototype;


/*
 * Start the retriever, and immediately do an update request.
 */
p.start=function() {
	if (AccountUpdateRetriever._running) {
		AccountUpdateRetriever.stop();
	}
	AccountUpdateRetriever._running=true;
	AccountUpdateRetriever.requestUpdate(); // this kicks off update checks every updateIntervalMillis ms; we don't set an interval here, rather, after the successful or failed call to this method, a setTimeout is called for updateIntervalMillis from now, until stop() is called.
};

/*
 * Stop the retriever, cancel any timeouts for new update requests, and ignore future responses until start() is called again.
 */
p.stop=function() {
	AccountUpdateRetriever._running=false;
	AccountUpdateRetriever.lastUpdateDataReceived={};
	AccountUpdateRetriever.lastUpdateIdReceived=0;
	if (AccountUpdateRetriever.requestUpdateTimeout!=null) {
		clearTimeout(AccountUpdateRetriever.requestUpdateTimeout);
		AccountUpdateRetriever.requestUpdateTimeout=null;
	}
};



/**
 * @param singleCall optional, default is false.  If true: a) this update is requested synchronously; b) it is requested regardless of the value of AccountUpdateRetriever._running; c) it does not set or clear any timeouts related to other calls; d) it displays a "please wait..." modal.  A true value here is thus not meant for "regular" updates that occur on a scheduled basis; it is meant for times when we want to force an update.
 * Perform an update request now; if there are outstanding requests for updates (in the form of timeouts), cancel them.  If there are updates (or specifically, if the update data differs from the stored data), the TunnelMgr will be updated, and then the UI told to update the tunnel list (if it's visible).
 */
p.requestUpdate=function(singleCall) {
	if (!UiMgr.isAcctLoggedIn()) { // no account logged on; nothing to update, and no point in requesting future ones until we are logged in:
		AccountUpdateRetriever.stop();
		return;
	}
	if (typeof singleCall=="undefined") {
		singleCall=false;
	}
	AccountUpdateRetriever.lastCallWasSingleCall=singleCall;
	if (!singleCall && AccountUpdateRetriever.requestUpdateTimeout!=null) {
		clearTimeout(AccountUpdateRetriever.requestUpdateTimeout);
		AccountUpdateRetriever.requestUpdateTimeout=null;
	}
	if (singleCall || AccountUpdateRetriever._running) {
		if (CommMgr.clientIsOnline()) {
			var map={};
			map.lastUpdateIdReceived=AccountUpdateRetriever.lastUpdateIdReceived;

			var asynchronous=!singleCall;
			CommMgr.initiateAjaxRequest(
					CommMgr.getControlServer(), 
					UiMgr.handlerKeys.CHECK_FOR_ACCOUNT_UPDATES, 
					map, 
					asynchronous, 
					AccountUpdateRetriever.processUpdateResponse, 
					function(response, textStatus, errorThrown){CommMgr.ajaxGlobalErrorCallback(response, textStatus, errorThrown);}, 
					15000
			);
			if (singleCall) {
				DialogMgr.closeBlockingModalDialog();
			}

		}
		if (!singleCall) {
			AccountUpdateRetriever.requestUpdateTimeout=setTimeout(AccountUpdateRetriever.requestUpdate, AccountUpdateRetriever.updateIntervalMillis);
		}
	}
	var tunnelsExist=(TunnelMgr.getTunnelsCount()>0);
	$("#tunnelListNoTunnelsMsg")[tunnelsExist?"hide":"show"]();
	$("#tunnelListPlusMinusButtons")[!tunnelsExist?"hide":"show"]();
	
	
};

p.processUpdateResponse=function(response) {
	var sessionExists=CommMgr.sessionExistsOnServerCheck(response);
	if (sessionExists) {
		if ((!AccountUpdateRetriever.lastCallWasSingleCall && !AccountUpdateRetriever._running) || UiMgr.destroyingTunnel) {
			return;
		}
		//console.info("calling setCurrentSessionDataFromResponse() from processUpdateResponse()...");
		var responseReceived=CommMgr.setCurrentSessionDataFromResponse(response);
		if (responseReceived && response.success) {
			AccountUpdateRetriever.lastUpdateIdReceived=response.updateId;
			var tunnelMgrChanges=AccountUpdateRetriever.reflectLatestAcctDataToTunnelMgr(response.updateData);
			if (tunnelMgrChanges) {
				AccountUpdateRetriever.notifyTunnelMgrChangesToUI();
				AccountUpdateRetriever.lastUpdateDataReceived=response.updateData;
			}
		}		
	}
};



/*
 * Updates the TunnelMgr with new data if it's changed; returns true if we updated the TunnelMgr, false if not
 */
p.reflectLatestAcctDataToTunnelMgr=function(updateData) {
	if (AccountUpdateRetriever.lastUpdateDataReceived=={} || AccountUpdateRetriever.getUpdateDataHash(AccountUpdateRetriever.lastUpdateDataReceived)!=AccountUpdateRetriever.getUpdateDataHash(updateData)) {
		TunnelMgr.refresh(updateData);
		return true;
	}
	return false;
};



p.getUpdateDataHash=function(updateData) {
	var hash="";
	var tunnelData
	for (var sessionTunnelId in updateData) {
		hash+=sessionTunnelId+":"
		tunnelData=updateData[sessionTunnelId];
		for (var key in tunnelData) {
			hash+=key+"="+tunnelData[key]+"&";
		}
		hash+="~";
	}
	return hash;
};


p.notifyTunnelMgrChangesToUI=function(updateData) {
	try {
		// for now, we're just going to redraw the entire list wrapper div based on the current contents of the TunnelMgr; this
		// is not efficient, in several ways, and needs to be changed:
		var tunnelsMap=TunnelMgr.getTunnelIdsToTunnelsMap();
		var ids=tunnelsMap.keysArray();
		var html="";
		var templateCode;
		if (window.mobileApp) {
			templateCode=$("#mobileTunnelListItem_template").html();
		}
		else {
			templateCode=$("#tunnelListItem_template").html();
		}
		var itemCode;
		for (var i=0; i<ids.length; i++) {
			itemCode=templateCode.replace(/\_\_ITEMID\_\_/g, ids[i]);
			html+=itemCode;
		}
		var listWrapperTopLevelBaseSelector="#";
		if (window.mobileApp) {
			listWrapperTopLevelBaseSelector+="mobileTunnelListWrapper";
		}
		else {
			listWrapperTopLevelBaseSelector+="tunnelListWrapper";		
		}
		$(listWrapperTopLevelBaseSelector).html(html);
		// make the overlays draggable:
		if (window.mobileApp) {
			$(".draggableOverlay").draggable({axis: "x", containment: [(TunnelMgr.REQUIRED_TUNNEL_LIST_ENTER_TUNNEL_ITEM_DRAG_PX+20)*-1,0,(TunnelMgr.REQUIRED_TUNNEL_LIST_FOR_DELETE_REVEAL_ITEM_DRAG_PX+1),0], revert:false, scroll:false, stop: function(event, ui) {TunnelMgr.tunnelListItemDragStop(event, ui);} });	
		}
		
		// now we've written the html, and have to set it up for each Tunnel in the list using jquery selectors:
		var selectorBase;
		var tunnel;
		var id;
		var showConfName;
		for (var i=0; i<ids.length; i++) {
			id=ids[i];
			selectorBase=listWrapperTopLevelBaseSelector+" #tunnelListItem"+id+" #draggableOverlay"+id+" ";
			tunnel=tunnelsMap.get(id);
			showConfName=(tunnel.confidantName!=UiMgr.unregisteredConfidantIdentifier);

			// set the orange circle to on or off depending on whether there are new, unread messages:
			$(selectorBase+"#circleIcon"+id+"On")[tunnel.hasUnreadMessages?"show":"hide"]();
			$(selectorBase+"#circleIcon"+id+"Off")[!tunnel.hasUnreadMessages?"show":"hide"]();
			// set the tunnel name and confidant's name:
			$(selectorBase+"#tunnelName"+id).html(tunnel.name.replace(/\s/g, "&nbsp;"));
			$(selectorBase+"#confName"+id).html(tunnel.confidantName.replace(/\s/g, "&nbsp;"));
			if (!professionalSite) {
				$(selectorBase+"#confNameInParens"+id)[showConfName?"show":"hide"]();				
			}
			else if (mobileApp && professionalSite) {
				$(selectorBase+"#confNameInParens"+id).hide();	
			}
			else {
				$(selectorBase+"#timeSpent"+id).html(tunnel.timeSpent);
			}
			
			// set the style of the tunnel/confidant names depending on if they confidant is online or not:
			$(selectorBase+"#tunnelNameConfName"+id).removeClass("confOffline confOnline").addClass(tunnel.confidantOnline?"confOnline":"confOffline");
		}
		var tunnelsExist=(ids.length>0);
		$("#tunnelListNoTunnelsMsg")[tunnelsExist?"hide":"show"]();

		$(".genericButton").button(); // style the buttons we've just written	
		DialogMgr.closeBlockingModalDialog(); // will have been open if there was an error on the last call
	}
	catch (e) {
		// if an error is thrown here, it's likely because client list is out of sync with server list because a confidant
		// just deleted a tunnel (not likely to happen, but it will happen sometimes).  So, just try again soon:
		DialogMgr.openBlockingModalDialog("Please wait...");
		setTimeout(function() {
			AccountUpdateRetriever.notifyTunnelMgrChangesToUI(updateData);
		}, 250);
	}
	
};


function allPropsDEBUG(obj) {
	var s="";
	for (var prop in obj) {
		s+=prop+"="+obj[prop]+"` "
	}
	return s;
}


var AccountUpdateRetriever=new _AccountUpdateRetriever();