


function _CommMgr () {
	if (typeof Map == "undefined") {
		throw new Error("CommMgr requires Map to function.");
	}
	this.serversMap=new Map();
	this.lastResponseReceivedTime=0; // 0 indicates no responses received yet
	this.defaultAjaxResponseTimeoutMillis=10000;
	this.asyncAjaxFileUploadTimeoutMillis=10000;
	this.failedAjaxClientRequestIds=new Map(); // failed ajax request IDs as keys; values are null (this acts as a Set, in other words, not a Map)
	this.initialSessionEstablished=false;
}

var p=_CommMgr.prototype;


p.addServer=function(id, server) {
	CommMgr.serversMap.put(id, server);
};

p.getServer=function(id) {
	return CommMgr.serversMap.get(id);
};

p.noop=function() {
};

p.resetLastResponseReceivedTime=function() {
	CommMgr.lastResponseReceivedTime=new Date().getTime();
};

p.getControlServer=function() {
	return CommMgr.getServer("controlServer");
};



/**
 * 
 * @param {Object} server A Server object representing the server we're submitting the request to
 * @param {Object} handlerKey A handler key which will be recognized by the Server
 * @param {Object} dataMap Data to be passed with the request (pass {} (empty object) if no data to pass)
 * @param {Object} asynchronous Boolean true if asynchronous, false if synchronous (if false, this function returns the ajax response)
 * @param {Object} successFunction A Function to be called on success (if null, CommMgr.noop is used); param is ignored if asynchronous is false
 * @param {Object} errorFunction A Function to be called on error (if null, CommMgr.noop is used); param is ignored if asynchronous is false
 * @param {Object} timeoutMillis Override the UiMgr.defaultAjaxResponseTimeoutMillis timeout with this optional param
 */ 
p.initiateAjaxRequest=function(server, handlerKey, dataMap, asynchronous, successFunction, errorFunction, timeoutMillis, retryAfterBadRequestCount) {
	var ajaxResponse=null;
	try {
		if (typeof server=="undefined" || server==null) {
			throw new Error("server is required.");
		}
		if (typeof handlerKey=="undefined" || handlerKey==null) {
			// this is an application error which we'll never see once we've fixed it the first time, so let's tell the
			// user (that is, the developer) aggressively:
			alert("The handlerKey you passed to initiateAjaxRequest was not defined; cannot continue.  About to throw Error...");
			throw new Error("handlerKey is required; the server probably has not passed the key you are trying to use.");
		}
		if (typeof dataMap=="undefined" || dataMap==null) {
			throw new Error("dataMap is required.");
		}
		if (typeof asynchronous=="undefined" || asynchronous==null) {
			throw new Error("asynchronous (true|false) param is required.");
		}
		if (typeof successFunction=="undefined" || successFunction==null) {
			successFunction=CommMgr.setCurrentSessionDataFromResponse;
		}
		if (typeof errorFunction=="undefined" || errorFunction==null) {
			errorFunction=CommMgr.ajaxGlobalErrorCallback;
		}

		if (typeof timeoutMillis=="undefined" || timeoutMillis==null) {
			timeoutMillis=UiMgr.defaultAjaxResponseTimeoutMillis;
		}
		if (typeof timeoutMillis=="undefined" || timeoutMillis==null || isNaN(timeoutMillis)) {
			timeoutMillis=(asynchronous?5000:500);
		}
		if (typeof retryAfterBadRequestCount=="undefined" || retryAfterBadRequestCount==null || isNaN(retryAfterBadRequestCount)) {
			retryAfterBadRequestCount=0;
		}
		
		if (retryAfterBadRequestCount>20) {
			throw new Error("Retried bad-request (err 400 response) 20 times; giving up.");
		}
		
		//console.info("Initiating AJAX request for "+handlerKey+" handler ("+asynchronous+")");

		
		// we don't do a session-established-and-online check for establish-session 
		// calls (because we wouldn't have a session yet) or for updates (since they're
		// so frequent), or for no-op calls.  Also note that we reference the handler-key name as
		// window.ESTABLISH_SESSION_HANDLER_KEY because UiMgr.handlerKeys hasn't
		// been initialized before the initial session has been established.
		if (handlerKey!=window.ESTABLISH_SESSION_HANDLER_KEY && handlerKey!=UiMgr.handlerKeys.CHECK_FOR_UPDATES && handlerKey!=UiMgr.handlerKeys.NO_OP && !CommMgr.onlineAndInitialSessionEstablishedCheck()) {
			return; // if we've never established a session, and just failed to do so, the user will have been alerted to this, and now we'll get out of here.
		}
		

		
		var completeFunction=CommMgr.resetLastResponseReceivedTime;
		dataMap.handlerKey=handlerKey;
		dataMap.requestedContentType="text/json";
		dataMap.ajaxClientRequestId=new Date().getTime()+"_"+Math.round(Math.random()*100000000); // an id for this request which the server will pass back; allows us to ignore timed-out requests
		dataMap.tempPassphraseSecondHalfActsAsPin=UiMgr.tempPassphraseSecondHalfActsAsPin; // This will always be true; but older clients won't pass it at all, and the AJAX APIs expect both.  Note that not all handlers look for this param, but we're including it here because it can't hurt.
		dataMap.sc=window.skinCode;
		if (window.mobileApp) {
			dataMap[window.mobileAppBooleanParamName]=true;
			// app doesn't support cookies, so we have to pass this this way:
			dataMap.mobileMultiSessionClientIdentifier=UiMgr.mobileMultiSessionClientIdentifier;
		}
		//alert("window.mobileApp="+window.mobileApp);
		var urlSessionAppender="";
		if (server.sessionId!=null) {
			urlSessionAppender=";"+UiMgr.sessionPathParamName+"="+server.sessionId;
		}
		var startRequestTime=new Date().getTime();
		var response=null;
		var cacheBuster="?cBust="+new Date().getTime(); // double protection against caching
		var ajaxUrl=server.ajhUrl+urlSessionAppender+cacheBuster;
		
		// always try again after an error-400 (bad-request), for synchronous requests only:
		var statusCodeMappingsSynchronous={
			400:function() {
				retryAfterBadRequestCount++;
				console.warn("*** 400/bad-request code received (handlerKey="+handlerKey+"; retryAfterBadRequestCount="+retryAfterBadRequestCount+"); re-POSTing...");
				return CommMgr.initiateAjaxRequest(server, handlerKey, dataMap, asynchronous, successFunction, errorFunction, timeoutMillis, retryAfterBadRequestCount);
			}
		};
	
		//console.info("about to do asynchronous="+asynchronous+" AJAX req with dataMap="+DEBUGobjToString(dataMap));

		
		if (asynchronous) {
			ajaxResponse=$.ajax({cache:false, type:"post", dataType:"json", timeout:timeoutMillis, url: ajaxUrl, data: dataMap, async: true, success: successFunction, error: errorFunction, complete: completeFunction}).fail(CommMgr.asyncAjaxFail(""+dataMap.ajaxClientRequestId));		
		}
		else {
			var start=new Date().getTime();
			ajaxResponse=$.ajax({statusCode:statusCodeMappingsSynchronous, cache:false, type:"post", dataType:"json", timeout:timeoutMillis, url: ajaxUrl, data: dataMap, async: false});
			CommMgr.lastResponseReceivedTime=new Date().getTime();
			//console.info("synchronous AJAX req handler="+handlerKey+" took "+(CommMgr.lastResponseReceivedTime-start)+"ms, about to eval()...");
			response=eval("("+ajaxResponse.responseText+")");
			//console.info("calling setCurrentSessionDataFromResponse() from initiateAjaxRequest()...");
			var responseReceived=CommMgr.setCurrentSessionDataFromResponse(response, server, startRequestTime);
			
			if (responseReceived && response.errorMsg!=null) {
				throw new Error(response.errorMsg);
			}
			return response;
		}
	}
	catch (__ee) {
		//alert("Error thrown for hk '"+handlerKey+"'; ajaxResponse.responseText (if any) was: "+(ajaxResponse==null?"[null ajaxResponse]":ajaxResponse.responseText));
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};



p.onlineAndInitialSessionEstablishedCheck=function() {
	if (mobileApp && !CommMgr.clientIsOnline()) {
		// we only do an online check for mobile, not desktop
		DialogMgr.alert("You must be online to do that!  Please make sure you have a connection, then try again.");
		return false;
	}
	if (!CommMgr.initialSessionEstablished) {
		var sessionEstablished=CommMgr.establishSession(CommMgr.getControlServer());
		if (!sessionEstablished) {
			DialogMgr.alert("You are either not online or your internet connection is too slow; please check and try again.");
			return false;
		}
		// we now have a session (and more important, a multiSession identifier, either as a cookie (web) or JS var (app), so:
		CommMgr.initialSessionEstablished=true;
		return true;
	}
	return true;
};



p.clientIsOnline=function() {
	try {
		if (window.mobileApp && MobileDevice.isOnlineStatusKnown()) {
			// if we're an app and the device knows whether it's online or not...
			return MobileDevice.isOnline();
		}
		else {
			// else we return true; this is really only an issue for mobile app:
			return true;
			/*
			// ...otherwise we're either non-app, or app but we don't yet know if we're online (because neither the online or offline
			// events have fired, particularly possible with iOS), so we'll make an AJAX call and see what happens:
			try {
				var response=CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.NO_OP, {}, false, function(){}, function(){}, 500);
				return (response.success);		
			}
			catch (e) {
				return false;
			}
			*/
		}
	}
	catch (__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}

};


p.defaultErrorCallback=function(requestId) {
	DialogMgr.closeBlockingModalDialog(); DialogMgr.alert("There was a problem; please make sure you have an internet connection. If you have this problem repeatedly, please contact Tunnel X.");};


p.asyncAjaxFail=function(requestId) {
	CommMgr.failedAjaxClientRequestIds.put(requestId);
	CommMgr.cleanUpFailedAjaxClientRequestIds(500); // don't let this Map get larger than 500 entries; could become a memory leak.
};


p.cleanUpFailedAjaxClientRequestIds=function(maxLength) {
	// iterate over the map, removing the first mapping (which is also the oldest) until the map is an acceptable size:
	while (CommMgr.failedAjaxClientRequestIds.size()>maxLength) {
		CommMgr.failedAjaxClientRequestIds.removeFirst();
	}
};



p.ajaxGlobalErrorCallback=function(response, textStatus, errorThrown) {
	/*
	var props="[";
	for (var prop in response) {
		props+=prop+"="+response[prop]+"; ";
	}
	props+="]";
	alert("in ajaxGlobalErrorCallback(), props="+props+"; textStatus="+textStatus+"; errorThrown="+errorThrown+"");
	
	*/
	/*
	var hk=null;
	var ajaxClientRequestId=null;
	var response=null;
	try {
		response=eval("("+ajaxResponse.responseText+")");
	}
	catch (e) {}
	if (response!=null) {
		if (typeof response._handlerKey!="undefined") {
			hk=response._handlerKey;
		}
		if (typeof response._handlerKey!="undefined") {
			ajaxClientRequestId=response._ajaxClientRequestId;
		}
	}
	alert("in ajaxGlobalErrorCallback(), response="+response+"; hk="+hk+"; ajaxClientRequestId="+ajaxClientRequestId+"; textStatus="+textStatus+"; errorThrown="+errorThrown+"");
	*/
};




//returns false if there was no response to parse
p.setCurrentSessionDataFromResponse=function(response, server, startRequestTime) {
	//console.info("in setCurrentSessionDataFromResponse()...");
	if (typeof response=="undefined" || response==null) {
		//console.info("returning false from setCurrentSessionDataFromResponse(); response="+response);
		return false; // failed to get response from server
	}
	var hk=response._handlerKey;
	var ajaxClientRequestId=response._ajaxClientRequestId;
	if (!CommMgr.failedAjaxClientRequestIds.containsKey(ajaxClientRequestId)) {
		//if (hk!="getUpdates") {
		//	alert("In setCurrentSessionDataFromResponse(), hk="+hk);
		//}
		if (typeof server=="undefined") { 
			server=null;
		}
		if (server==null) {
			server=CommMgr.getControlServer();
		}
		if (typeof response._jsi!="undefined" && response._jsi!=null && response._jsi!="") {
			server.sessionId=response._jsi;
		}
		server.sessionExists=(server.sessionId!=null);
	}
	//console.info("returning true from setCurrentSessionDataFromResponse()");
	return true;
	
};



/**
 * Returns true if a session was established, false otherwise (because we're offline probably) -- here we also check with the
 * servver to make sure the client version we're using is up to date (and that we're not using a tablet); if we're not using
 * a supported device, we show the user a blocking modal, wwhich disallows them from doing anyhting else until they upgrade.
 */
p.establishSession=function(server) {
	try {
		var dataMap={buildNumber:window.uiMgrProps.webappBuildNumber};
		var response=CommMgr.initiateAjaxRequest(server, window.ESTABLISH_SESSION_HANDLER_KEY, dataMap, false, null, null, 30000);	
		if (mobileApp && typeof response.clientVersionErrorMessage!="undefined" && response.clientVersionErrorMessage!=null) {
			DialogMgr.openBlockingModalDialog("<br/>"+response.clientVersionErrorMessage);
			UiMgr.permanentlyDisableFooterLinks=true;
			// hide these links delayed, since they'll get shown otherwise; little ugly, but we'll live:
			setTimeout(function() {
				UiMgr.showFooterIcon("help", false);
				UiMgr.showFooterIcon("exit", false);
				// this will leave the only link in the footer as the logo; need that for branding, but it won't do anything when clicked.				
			}, 200);
			// Note: even though we won't let the user use the app, we wwill continue wwith the rest of the code before we return true,
			// because otherwwise the uer will get error messages rather than seeing the message we want them to see --
		}
		CommMgr.initialSessionEstablished=true;
		PinPad.disallowedPinSequencesListArray=response.disallowedPinSequencesList;
		PinPad.pinButtonTokensToDigitsMap=new Map(response.pinButtonTokensToDigitsMap);
		if (window.mobileApp) {
			UiMgr.mobileMultiSessionClientIdentifier=response.multiSessionClientIdentifier.replace(/\%2B/g, "+");
		}
		UiMgr.skinnedSiteData=response.skinnedSiteData; //skinnedSiteData props: see EstablishSession handler for most current list

		return true;
	}
	catch (__ee) {
		//alert("DEBUG: error in establishSession, returning false...");
		return false;
		//throw __ee;
	}
};

p.startAsynchronousAjaxFileUpload=function(server, handlerKey, qString) {
	//console.info("Starting async AJAX upload for "+handlerKey+" handler");
    var fileUploadFormObj=$("#fileUploadForm");
	try {
		if (typeof server=="undefined" || server==null) {
			throw new Error("server is required.");
		}
		if (typeof handlerKey=="undefined" || handlerKey==null) {
			throw new Error("handlerKey is required.");
		}
		if (typeof qString=="undefined" || qString==null) {
			qString="";
		}
	
		
		var urlSessionAppender="";
		if (server.sessionId!=null) {
			urlSessionAppender=";"+UiMgr.sessionPathParamName+"="+server.sessionId;
		}
		var ajaxFormId=fileUploadFormObj.attr("id");

		
	
		$("#"+ajaxFormId+" #handlerKey").val(handlerKey);
		$("#"+ajaxFormId+" #skinCode").val(window.skinCode);
		$("#"+ajaxFormId+" #requestedContentType").val("text/json");
		$("#"+ajaxFormId+" #timeMillis").val(new Date().getTime());
		$("#"+ajaxFormId+" #ajaxClientRequestId").val(new Date().getTime()+"_"+Math.round(Math.random()*100000000)); // an id for this request which the server will pass back; allows us to ignore timed-out requests
		//alert("upload: window.mobileApp="+window.mobileApp);
		
		if (window.mobileApp) {
			// tell the server we're a mobile app (needs to be on qstring because this value is needed before params are parsed:
			qString+=(qString.indexOf("?")==-1?"?":"&");
			qString+=window.mobileAppBooleanParamName+"=true";
			
			// mobile app doesn't support cookies, so we have to pass this value this way:
			$("#"+ajaxFormId+" #mobileMultiSessionClientIdentifier").val(UiMgr.mobileMultiSessionClientIdentifier);
		}
		

	
		// if we don't do this, we end up with multiple ajaxSubmits bound to this form, each of which fires when we submit().
		fileUploadFormObj.unbind("submit");
		
	    var options={
	    		url: server.ajhUrl+urlSessionAppender+qString, 
	    		clearForm: true,
	    		resetForm: true,
	    		timeout: CommMgr.asyncAjaxFileUploadTimeoutMillis,
	    		dataType: "json"
	    		};
		//alert("options: "+options);
		fileUploadFormObj.on("submit", function() { 
	        $(this).ajaxSubmit(options); 
	        return false; // return false to prevent standard browser submit
	    }); 
		if (!window.useCordovaFilePluginForUploads) {
			//alert("submitting (hk="+handlerKey+")...");
		    fileUploadFormObj.submit();
		    //alert("submitted.");
		}
		else {
			var fileUri=$("#cordovaFileUri").val();

			// we only need to set the fileName as the defaults are what we need
            var fileUploadOptions=new FileUploadOptions();
            fileUploadOptions.fileName = fileUri.substr(fileUri.lastIndexOf('/') + 1);

            // additional data to send along with the image
            var params={};
            params.handlerKey=handlerKey;
            params.requestedContentType="text/json";
            params.timeMillis=new Date().getTime();
            params.ajaxClientRequestId=new Date().getTime()+"_"+Math.round(Math.random()*100000000);
            params.mobileMultiSessionClientIdentifier=UiMgr.mobileMultiSessionClientIdentifier;
            if (handlerKey==UiMgr.handlerKeys.UPLOAD_MEDIA) {
            	params.filename=UiMgr.getFilenameOnly(fileUri);
            	params.fileSize=$("#cordovaFileSize").val();
            }
            fileUploadOptions.params=params;

            var ft=new FileTransfer();
            ft.upload(fileUri, encodeURI(options.url), null, null, fileUploadOptions);
            return false;
		}
	}
	catch (__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};




/**
 * Generates a Map of param name-value pairs to be send with requests that require a client nonce and hash
 * @param serverNonce The nonce generated by the server and sent to the client in an AJAX response
 * @param nonNonceValue Optional; generally a value like a PIN which we'll include in the hash.  Will be ignored if null or not passed.
 * @return Map of param name-value pairs, with names _cn (the client-generated nonce) and _ch (a hash generated exactly as com.tunnelx.ajax.AbstractAjaxRequestHandler.validateNoncesHash(AbstractRequest request, String nonNonceValueInHashParamName)
 */
p.getNonceHashParams=function(serverNonce, nonNonceValue) {
	var nameValuePairs=new Map();
	if (typeof nonNonceValue=="undefined") {
		nonNonceValue=null;
	}
	var clientNonce=Crypto.generateRandomBits(256); // get a 256-bit nonce (as hex-encoded string)
	var valueToHash=new String(serverNonce);
	if (nonNonceValue!=null) { 
		valueToHash+=nonNonceValue;
	}
	// hash the string we've just built (just the server nonce, or the server nonce plus the nonNonceValue (a PIN, or whatever), salting with the clientNonce:
	var clientHash=Crypto.sha256HashAsHex(valueToHash, clientNonce);

	// Param names (map keys) here must correspond to names defined as constants in com.tunnelx.ajax.AbstractAjaxRequestHandler:
	nameValuePairs.put("_cn", clientNonce); // client nonce
	nameValuePairs.put("_ch", clientHash); // client hash
	return nameValuePairs;
};

/** Add params to the JavaScript object that gets passed to $.ajax() as the "data" value
 * @param nameValuePairsAsMap an instance of our Map object (see Map.js) containing name-value pairs to be added
 * @param dataObject The JavaScript object to which to add the Map's name-value pairs
 * 
 */
p._addParamsToAjaxDataObject=function(nameValuePairsAsMap, dataObject) {
	var mapKeys=nameValuePairsAsMap.keysArray();
	for (var i=0; i<mapKeys.length; i++) {
		dataObject[mapKeys[i]]=nameValuePairsAsMap.get(mapKeys[i]);
	}
};

/** Set nonce/hash-related form-element values on the file-upload form
 * @param nameValuePairsAsMap an instance of our Map object (see Map.js) containing name-value pairs to be added
 * 
 */
p._setValuesOnFileUploadForm=function(nameValuePairsAsMap) {
	var fileUploadFormSelector="#fileUploadForm";
	var mapKeys=nameValuePairsAsMap.keysArray();
	for (var i=0; i<mapKeys.length; i++) {
		$(fileUploadFormSelector+" #"+mapKeys[i]).val(nameValuePairsAsMap.get(mapKeys[i]));
	}
};








/**
 * Retrieve a handlerKey-specific nonce from the server (the server will store it, and await a request for this handler which contains this 
 * nonce and a hash derived from it.  If this function fails to retrieve a nonce (e.g. because there's no internet connection after 10 seconds),
 * it will return null.
 * @return A nonce, or null if the server couldn't be contacted or didn't respond.
 */
p._getServerNonce=function(handlerKey, retryCount) {
	/*
	if (typeof retryCount=="undefined") {
		retryCount=0;
	}
	try {
		var response=CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.GET_SERVER_NONCE, {nonceHandlerKey:handlerKey}, false, null, null, 30000);
		//alert("response="+response);
		var nonce=response.nonce;
		if (nonce!=null && typeof nonce!="undefined") { // we don't need to check for valid length or similar here; if the nonce is malformed, the handler to which this nonce is passed will catch that.
			return nonce;
		}
		else {
			throw new Error("nonce not found in response.");
		}
	}
	catch (e) {
		if (retryCount<3) {
			retryCount++;
			return CommMgr._getServerNonce(handlerKey, retryCount);
		}
		else {
			throw e;
		}
	}
	*/
};


p.addNonceAndHashToAjaxDataObject=function(handlerKey, dataObject, nonNonceHashValue) {
	/*
	var serverNonce=null;
	try {
		serverNonce=CommMgr._getServerNonce(handlerKey);
	}
	catch (e) {
		console.error("Couldn't retrieve serverNonce from server; about to rethrow e.message="+e.message);
		throw e;
	}
	var nonceHashParamsMap=CommMgr.getNonceHashParams(serverNonce, nonNonceHashValue);
	CommMgr._addParamsToAjaxDataObject(nonceHashParamsMap, dataObject);
	*/
};

p.addNonceAndHashToFileUploadForm=function(handlerKey, nonNonceHashValue) {
	/*
	var serverNonce=null;
	try {
		serverNonce=CommMgr._getServerNonce(handlerKey);
	}
	catch (e) { 
		console.error("Couldn't retrieve serverNonce from server; about to rethrow e.message="+e.message);
		throw e;
	}
	var nonceHashParamsMap=CommMgr.getNonceHashParams(serverNonce, nonNonceHashValue);
	CommMgr._setValuesOnFileUploadForm(nonceHashParamsMap);
	*/
};


p.sessionExistsOnServerCheck=function(response) {
	if (typeof response!="undefined" && response!=null && typeof response.acctNotFoundForSession!="undefined" && response.acctNotFoundForSession==true) {
		//console.info("in sessionExistsOnServerCheck(), response.acctNotFoundForSession is true...");
		// The server responded, but doesn't know about our session anymore.  Once we are able to "resurrect" sessions, this
		// will not be a problem, but for now, we need to stop continuing to make requests; this taxes the server and wastes
		// bandwidth and clogs logs and is in no way recoverable, so:
		
		// stop the update requests that we ping for when we're in a tunnel:
		window.clearTimeout(MessengerMgr.updateCheckTimeout);
		// stop the account updates we ping for when we're logged in but not in a tunnel:
		AccountUpdateRetriever.stop();
		DialogMgr.alert("Your session has timed out; please log in again.", "Okay", function() {
			// Go to the homepage (web) or show the login PIN pad (mobile; goToHomepageAndResetUi() takes mobile users to the "homepage" but effectively that means displaying the PIN pad, since this user was logged in, and so has a key stored in localStorage)
			UiMgr.goToHomepageAndResetUi(true);
		});
		return false;
	}	
	return true;
};



function Server (ajhUrl, serverType) {
	this.ajhUrl=ajhUrl;
	this.serverType=serverType;
	this.sessionId=null;
	this.sessionExists=false;
}

var CommMgrConstants={};
CommMgrConstants.SERVER_TYPE_CONTROL="ctrl";

function DEBUGobjToString(obj) {
	var s="[";
	for (var prop in obj) {
		if ((typeof obj[prop]).toLowerCase()!="function") {
			s+=prop+":"+obj[prop]+" ("+(typeof obj[prop])+"),";			
		}
	}
	return s+"]";
}

Server[P].toString=function() {
	var s="[";
	s+="ajhUrl="+this.ajhUrl+"; ";
	s+="sessionId="+this.sessionId+"; ";
	s+="sessionExists="+this.sessionExists+"; ";
	s+="]";
	return s;
};




var CommMgr=new _CommMgr();

