/*
 A note about client-side crypto on Tunnel X: its use is limited to hashing nonces (for which we use the Stanford Javascript 
 Crypto Library) and generating client nonces.  (If you don't know what a nonce is, you probably don't care whether or not Tunnel 
 X uses JavaScript cryptography or not anyway.)  None of this JavaScript-based crypto is at the core of what makes Tunnel X
 secure; rather, it adds a layer of security which, at worst (if compromised), merely fails to provide an extra measure of 
 security.  Thus is can only enhance the overall security of the system.
 
 We do not otherwise use client-side crypto (e.g., for encryption of any type), aside obviously from TLS.  It is our strong
 implementation of TLS which primarily provides for the secrecy of data in transit.
 */

if (typeof window.sjcl == "undefined") {
	throw new Error("The Crypto class requires the SJCL (sjcl.js) library.");
}

function _Crypto () {
	this.MAX_ALLOWED_PARANOIA=7;
	this.PARANOIA=this.MAX_ALLOWED_PARANOIA;
	this.initted=false;
}

var p=_Crypto.prototype;


p.isPrngReady=function() {
	return Crypto.sjclPrng.isReady(Crypto.PARANOIA);
};


p.init=function() {
	Crypto.sjclPrng=sjcl.random;
	Crypto.sjclPrng.startCollectors();
	Crypto.initted=true; // this does NOT mean that the PRNG is ready; just that the object has been initialized.
};

/**
 * Salts and SHA-256-hashes a string and hex-encoded salt in a way identical to com.tunnelx.encryption.CryptoUtils.sha256HashAsHexLikeWebClient(String plaintextStr, String saltStrHex).
 */
p.sha256HashAsHex=function(plaintextStr, saltStrHex) {
	var start=new Date().getTime();
	if (!Crypto.initted) {
		throw new Error("You must call Crypto.init() before using the Crypto class in any way.");
	}
	if (!Crypto.isPrngReady()) {
		throw new Error("Crypto.sha256HashAsHex() was called before Crypto.isPrngReady() was true.");
	}
	var ret=sjcl.codec.hex.fromBits(sjcl.hash.sha256.hash(plaintextStr+saltStrHex)).toUpperCase();  
	//console.info("Crypto.sha256HashAsHex() took "+(new Date().getTime()-start)+"ms");
	return ret;
};

/**
 * @param numBits The number of random bits to return.  Must be a multiple of 32 or error is thrown.
 * @return hexadecimal representation of random bits
 */
p.generateRandomBits=function(numBits) {
	var start=new Date().getTime();
	if (!Crypto.initted) {
		throw new Error("You must call Crypto.init() before using the Crypto class in any way.");
	}
	if (!Crypto.isPrngReady()) {
		throw new Error("Crypto.sha256HashAsHex() was called before Crypto.isPrngReady() was true.");
	}
	numBits=parseInt(numBits);
	var numWords=numBits/32;
	if (parseFloat(numWords)!=parseInt(numWords)) {
		throw new Error("numBits ("+numBits+") must be divisible by 32.");
	}
	var ret=sjcl.codec.hex.fromBits(Crypto.sjclPrng.randomWords(numWords, Crypto.PARANOIA)).toUpperCase();	
	//console.info("Crypto.generateRandomBits() took "+(new Date().getTime()-start)+"ms");	
	return ret;
};


p.intToHex=function(n) {
	var start=new Date().getTime();
	if (!Crypto.initted) {
		throw new Error("You must call Crypto.init() before using the Crypto class in any way.");
	}
	if (n<0) {
		throw new Error("n must be integer >=0");
	}
	var s=Number(n).toString(16); 
	var ret=s.length==1? "0"+s:s; 
	//console.info("Crypto.intToHex() took "+(new Date().getTime()-start)+"ms");	
	return ret;
};



var Crypto=new _Crypto();

